# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/types/transaction_log.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/types/transaction_log.proto',
  package='iotextypes',
  syntax='proto3',
  serialized_options=b'\n\"com.github.iotexproject.grpc.typesP\001Z5github.com/iotexproject/iotex-proto/golang/iotextypes',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n!proto/types/transaction_log.proto\x12\niotextypes\"\xfa\x01\n\x0eTransactionLog\x12\x12\n\nactionHash\x18\x01 \x01(\x0c\x12\x17\n\x0fnumTransactions\x18\x02 \x01(\x04\x12<\n\x0ctransactions\x18\x03 \x03(\x0b\x32&.iotextypes.TransactionLog.Transaction\x1a}\n\x0bTransaction\x12\r\n\x05topic\x18\x01 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x02 \x01(\t\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x11\n\trecipient\x18\x04 \x01(\t\x12,\n\x04type\x18\x05 \x01(\x0e\x32\x1e.iotextypes.TransactionLogType\";\n\x0fTransactionLogs\x12(\n\x04logs\x18\x01 \x03(\x0b\x32\x1a.iotextypes.TransactionLog*\x87\x02\n\x12TransactionLogType\x12\x18\n\x14IN_CONTRACT_TRANSFER\x10\x00\x12\x13\n\x0fWITHDRAW_BUCKET\x10\x01\x12\x11\n\rCREATE_BUCKET\x10\x02\x12\x15\n\x11\x44\x45POSIT_TO_BUCKET\x10\x03\x12\x18\n\x14\x43\x41NDIDATE_SELF_STAKE\x10\x04\x12\x1e\n\x1a\x43\x41NDIDATE_REGISTRATION_FEE\x10\x05\x12\x0b\n\x07GAS_FEE\x10\x06\x12\x13\n\x0fNATIVE_TRANSFER\x10\x07\x12\x1d\n\x19\x44\x45POSIT_TO_REWARDING_FUND\x10\x08\x12\x1d\n\x19\x43LAIM_FROM_REWARDING_FUND\x10\tB]\n\"com.github.iotexproject.grpc.typesP\x01Z5github.com/iotexproject/iotex-proto/golang/iotextypesb\x06proto3'
)

_TRANSACTIONLOGTYPE = _descriptor.EnumDescriptor(
  name='TransactionLogType',
  full_name='iotextypes.TransactionLogType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IN_CONTRACT_TRANSFER', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WITHDRAW_BUCKET', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CREATE_BUCKET', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEPOSIT_TO_BUCKET', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CANDIDATE_SELF_STAKE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CANDIDATE_REGISTRATION_FEE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GAS_FEE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NATIVE_TRANSFER', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEPOSIT_TO_REWARDING_FUND', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CLAIM_FROM_REWARDING_FUND', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=364,
  serialized_end=627,
)
_sym_db.RegisterEnumDescriptor(_TRANSACTIONLOGTYPE)

TransactionLogType = enum_type_wrapper.EnumTypeWrapper(_TRANSACTIONLOGTYPE)
IN_CONTRACT_TRANSFER = 0
WITHDRAW_BUCKET = 1
CREATE_BUCKET = 2
DEPOSIT_TO_BUCKET = 3
CANDIDATE_SELF_STAKE = 4
CANDIDATE_REGISTRATION_FEE = 5
GAS_FEE = 6
NATIVE_TRANSFER = 7
DEPOSIT_TO_REWARDING_FUND = 8
CLAIM_FROM_REWARDING_FUND = 9



_TRANSACTIONLOG_TRANSACTION = _descriptor.Descriptor(
  name='Transaction',
  full_name='iotextypes.TransactionLog.Transaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='topic', full_name='iotextypes.TransactionLog.Transaction.topic', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='iotextypes.TransactionLog.Transaction.amount', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='iotextypes.TransactionLog.Transaction.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recipient', full_name='iotextypes.TransactionLog.Transaction.recipient', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='iotextypes.TransactionLog.Transaction.type', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=175,
  serialized_end=300,
)

_TRANSACTIONLOG = _descriptor.Descriptor(
  name='TransactionLog',
  full_name='iotextypes.TransactionLog',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='actionHash', full_name='iotextypes.TransactionLog.actionHash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='numTransactions', full_name='iotextypes.TransactionLog.numTransactions', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transactions', full_name='iotextypes.TransactionLog.transactions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_TRANSACTIONLOG_TRANSACTION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=300,
)


_TRANSACTIONLOGS = _descriptor.Descriptor(
  name='TransactionLogs',
  full_name='iotextypes.TransactionLogs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='logs', full_name='iotextypes.TransactionLogs.logs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=302,
  serialized_end=361,
)

_TRANSACTIONLOG_TRANSACTION.fields_by_name['type'].enum_type = _TRANSACTIONLOGTYPE
_TRANSACTIONLOG_TRANSACTION.containing_type = _TRANSACTIONLOG
_TRANSACTIONLOG.fields_by_name['transactions'].message_type = _TRANSACTIONLOG_TRANSACTION
_TRANSACTIONLOGS.fields_by_name['logs'].message_type = _TRANSACTIONLOG
DESCRIPTOR.message_types_by_name['TransactionLog'] = _TRANSACTIONLOG
DESCRIPTOR.message_types_by_name['TransactionLogs'] = _TRANSACTIONLOGS
DESCRIPTOR.enum_types_by_name['TransactionLogType'] = _TRANSACTIONLOGTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TransactionLog = _reflection.GeneratedProtocolMessageType('TransactionLog', (_message.Message,), {

  'Transaction' : _reflection.GeneratedProtocolMessageType('Transaction', (_message.Message,), {
    'DESCRIPTOR' : _TRANSACTIONLOG_TRANSACTION,
    '__module__' : 'proto.types.transaction_log_pb2'
    # @@protoc_insertion_point(class_scope:iotextypes.TransactionLog.Transaction)
    })
  ,
  'DESCRIPTOR' : _TRANSACTIONLOG,
  '__module__' : 'proto.types.transaction_log_pb2'
  # @@protoc_insertion_point(class_scope:iotextypes.TransactionLog)
  })
_sym_db.RegisterMessage(TransactionLog)
_sym_db.RegisterMessage(TransactionLog.Transaction)

TransactionLogs = _reflection.GeneratedProtocolMessageType('TransactionLogs', (_message.Message,), {
  'DESCRIPTOR' : _TRANSACTIONLOGS,
  '__module__' : 'proto.types.transaction_log_pb2'
  # @@protoc_insertion_point(class_scope:iotextypes.TransactionLogs)
  })
_sym_db.RegisterMessage(TransactionLogs)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# The MIT License (MIT)
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from urllib.parse import urlparse

import grpc

from iotexetl.rpc.iotexapi import api_pb2
from iotexetl.rpc.iotexapi import api_pb2_grpc


class IotexRpc:

    def __init__(self, provider_uri, timeout=60):
        self.timeout = timeout
        channel = get_channel_from_uri_string(provider_uri)
        self.stub = api_pb2_grpc.APIServiceStub(channel)

    def get_raw_blocks(self, start_height, count):
        return self.stub.GetRawBlocks(
            api_pb2.GetRawBlocksRequest(startHeight=start_height, count=count, withReceipts=True), timeout=self.timeout)

    def get_block_metas(self, start_height, count):
        return self.stub.GetBlockMetas(api_pb2.GetBlockMetasRequest(
            byIndex=api_pb2.GetBlockMetasByIndexRequest(start=start_height, count=count)
        ), timeout=self.timeout)

    def get_transaction_logs(self, block_number):
        return self.stub.GetTransactionLogByBlockHeight(
            api_pb2.GetTransactionLogByBlockHeightRequest(blockHeight=block_number), timeout=self.timeout)

    def get_logs(self, block_number_batch):
        return self.stub.GetLogs(api_pb2.GetLogsRequest(filter=api_pb2.LogsFilter(),
                                                        byRange=api_pb2.GetLogsByRange(fromBlock=block_number_batch[0],
                                                                                       count=len(block_number_batch))),
                                 timeout=self.timeout)

    def get_chain_meta(self):
        return self.stub.GetChainMeta(api_pb2.GetChainMetaRequest(), timeout=self.timeout)


def get_channel_from_uri_string(provider_uri):
    uri = urlparse(provider_uri)
    if uri.scheme == 'grpcs':
        credentials = grpc.ssl_channel_credentials()
        channel = grpc.secure_channel(uri.netloc, credentials)
    elif uri.scheme == 'grpc':
        channel = grpc.insecure_channel(uri.netloc)
    else:
        raise ValueError(f'The uri scheme {uri.scheme} is not recognized. Use grpc:// or grpcs://')

    return channel



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import grpc


class IotexService(object):
    def __init__(self, iotex_rpc):
        self.iotex_rpc = iotex_rpc

    def get_genesis_block(self):
        return self.get_block(1)

    def get_latest_block(self):
        meta = self.iotex_rpc.get_chain_meta()
        return self.get_block(meta.chainMeta.height)

    def get_block(self, block_number):
        blocks = self.get_blocks([block_number])
        if len(blocks) == 0:
            return None
        else:
            return blocks[0]

    def get_blocks(self, block_number_batch):
        if not block_number_batch:
            return []
        response = self.iotex_rpc.get_raw_blocks(start_height=block_number_batch[0], count=len(block_number_batch))
        return response.blocks

    def get_block_metas(self, block_number_batch):
        if not block_number_batch:
            return []
        response = self.iotex_rpc.get_block_metas(start_height=block_number_batch[0], count=len(block_number_batch))
        return response.blkMetas

    def get_evm_transfers(self, block_number):
        try:
            response = self.iotex_rpc.get_evm_transfers(block_number)
            return response.blockEvmTransfers
        except grpc.RpcError as e:
            if e.code() != grpc.StatusCode.NOT_FOUND:
                raise
            return None

    def get_transaction_logs(self, block_number_batch):
        if not block_number_batch:
            return []
        for block_number in block_number_batch:
            response = self.iotex_rpc.get_transaction_logs(block_number)
            yield response

    def get_logs(self, block_number_batch):
        if not block_number_batch:
            return []
        response = self.iotex_rpc.get_logs(block_number_batch)
        return response.logs


import logging

from blockchainetl_common.jobs.exporters.console_item_exporter import ConsoleItemExporter
from blockchainetl_common.jobs.exporters.in_memory_item_exporter import InMemoryItemExporter
from iotexetl.enumeration.entity_type import EntityType
from iotexetl.jobs.export_blocks_job import ExportBlocksJob
from iotexetl.jobs.export_transaction_logs_job import ExportTransactionLogsJob

from iotexetl.streaming.item_id_calculator import ItemIdCalculator


class IotexStreamerAdapter:
    def __init__(
            self,
            iotex_rpc,
            item_exporter=ConsoleItemExporter(),
            batch_size=100,
            max_workers=5,
            entity_types=tuple(EntityType.ALL_FOR_STREAMING)):
        self.iotex_rpc = iotex_rpc
        self.item_exporter = item_exporter
        self.batch_size = batch_size
        self.max_workers = max_workers
        self.entity_types = entity_types
        self.item_id_calculator = ItemIdCalculator()

    def open(self):
        self.item_exporter.open()

    def get_current_block_number(self):
        meta = self.iotex_rpc.get_chain_meta()
        return meta.chainMeta.height

    def export_all(self, start_block, end_block):
        # Export blocks, actions and logs
        blocks, actions, logs = [], [], []
        if self._should_export(EntityType.BLOCK) or self._should_export(EntityType.ACTION) or self._should_export(EntityType.LOG):
            blocks, actions, logs = self._export_blocks(start_block, end_block)

        # Extract transaction logs
        transaction_logs = []
        if self._should_export(EntityType.TRANSACTION_LOG):
            transaction_logs = self._export_transaction_logs(start_block, end_block)

        logging.info('Exporting with ' + type(self.item_exporter).__name__)

        all_items = blocks + \
            actions + \
            logs + \
            transaction_logs

        self.calculate_item_ids(all_items)

        self.item_exporter.export_items(all_items)

    def _export_blocks(self, start_block, end_block):
        item_exporter = InMemoryItemExporter(item_types=[EntityType.BLOCK, EntityType.ACTION, EntityType.LOG])
        job = ExportBlocksJob(
            start_block=start_block,
            end_block=end_block,
            batch_size=self.batch_size,
            iotex_rpc=self.iotex_rpc,
            max_workers=self.max_workers,
            item_exporter=item_exporter,
            export_blocks=self._should_export(EntityType.BLOCK),
            export_actions=self._should_export(EntityType.ACTION),
            export_logs=self._should_export(EntityType.LOG),
        )
        job.run()
        blocks = item_exporter.get_items(EntityType.BLOCK)
        actions = item_exporter.get_items(EntityType.ACTION)
        logs = item_exporter.get_items(EntityType.LOG)
        return blocks, actions, logs

    def _export_transaction_logs(self, start_block, end_block):
        item_exporter = InMemoryItemExporter(item_types=[EntityType.TRANSACTION_LOG])
        job = ExportTransactionLogsJob(
            start_block=start_block,
            end_block=end_block,
            iotex_rpc=self.iotex_rpc,
            max_workers=self.max_workers,
            item_exporter=item_exporter)
        job.run()
        transaction_logs = item_exporter.get_items(EntityType.TRANSACTION_LOG)
        return transaction_logs

    def _should_export(self, entity_type):
        if entity_type == EntityType.BLOCK:
            return True

        if entity_type == EntityType.ACTION \
                or entity_type == EntityType.LOG \
                or entity_type == EntityType.TRANSACTION_LOG:
            return entity_type in self.entity_types

        raise ValueError('Unexpected entity type ' + entity_type)

    def calculate_item_ids(self, items):
        for item in items:
            item['item_id'] = self.item_id_calculator.calculate(item)

    def close(self):
        self.item_exporter.close()



from iotexetl.utils.string_utils import to_int


def map_receipt(receipt):
    return {
        'status': receipt.status,
        'height': receipt.blkHeight,
        'hash': receipt.actHash.hex(),
        'gas_consumed': to_int(receipt.gasConsumed),
        'contract_address': receipt.contractAddress
    }


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/types/election.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/types/election.proto',
  package='iotextypes',
  syntax='proto3',
  serialized_options=b'\n\"com.github.iotexproject.grpc.typesP\001Z5github.com/iotexproject/iotex-proto/golang/iotextypes',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1aproto/types/election.proto\x12\niotextypes\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xad\x01\n\x0e\x45lectionBucket\x12\r\n\x05voter\x18\x01 \x01(\x0c\x12\x11\n\tcandidate\x18\x02 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x0c\x12-\n\tstartTime\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x08\x64uration\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\r\n\x05\x64\x65\x63\x61y\x18\x06 \x01(\x08\x42]\n\"com.github.iotexproject.grpc.typesP\x01Z5github.com/iotexproject/iotex-proto/golang/iotextypesb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_ELECTIONBUCKET = _descriptor.Descriptor(
  name='ElectionBucket',
  full_name='iotextypes.ElectionBucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='voter', full_name='iotextypes.ElectionBucket.voter', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='candidate', full_name='iotextypes.ElectionBucket.candidate', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='iotextypes.ElectionBucket.amount', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='startTime', full_name='iotextypes.ElectionBucket.startTime', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='duration', full_name='iotextypes.ElectionBucket.duration', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='decay', full_name='iotextypes.ElectionBucket.decay', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=108,
  serialized_end=281,
)

_ELECTIONBUCKET.fields_by_name['startTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ELECTIONBUCKET.fields_by_name['duration'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['ElectionBucket'] = _ELECTIONBUCKET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ElectionBucket = _reflection.GeneratedProtocolMessageType('ElectionBucket', (_message.Message,), {
  'DESCRIPTOR' : _ELECTIONBUCKET,
  '__module__' : 'proto.types.election_pb2'
  # @@protoc_insertion_point(class_scope:iotextypes.ElectionBucket)
  })
_sym_db.RegisterMessage(ElectionBucket)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from iotexetl.mappers.receipt_mapper import map_receipt
from iotexetl.utils import string_utils, iotex_utils
from iotexetl.utils.string_utils import to_int


def map_action(raw):
    for action, receipt in zip(raw.block.body.actions, raw.receipts):
        action_dict = {}
        if action.core.WhichOneof('action') == 'transfer':
            action_dict = map_transfer(action)
        elif action.core.WhichOneof('action') == 'execution':
            action_dict = map_execution(action)
        elif action.core.WhichOneof('action') == 'startSubChain':
            action_dict = map_start_sub_chain(action)
        elif action.core.WhichOneof('action') == 'stopSubChain':
            action_dict = map_stop_sub_chain(action)
        elif action.core.WhichOneof('action') == 'putBlock':
            action_dict = map_put_block(action)
        elif action.core.WhichOneof('action') == 'createDeposit':
            action_dict = map_create_deposit(action)
        elif action.core.WhichOneof('action') == 'settleDeposit':
            action_dict = map_settle_deposit(action)
        elif action.core.WhichOneof('action') == 'createPlumChain':
            action_dict = map_create_plum_chain(action)
        elif action.core.WhichOneof('action') == 'terminatePlumChain':
            action_dict = map_terminate_plum_chain(action)
        elif action.core.WhichOneof('action') == 'plumPutBlock':
            action_dict = map_plum_put_block(action)
        elif action.core.WhichOneof('action') == 'plumCreateDeposit':
            action_dict = map_plum_create_deposit(action)
        elif action.core.WhichOneof('action') == 'plumStartExit':
            action_dict = map_plum_start_exit(action)
        elif action.core.WhichOneof('action') == 'plumChallengeExit':
            action_dict = map_plum_challenge_exit(action)
        elif action.core.WhichOneof('action') == 'plumResponseChallengeExit':
            action_dict = map_plum_response_challenge_exit(action)
        elif action.core.WhichOneof('action') == 'plumFinalizeExit':
            action_dict = map_plum_finalize_exit(action)
        elif action.core.WhichOneof('action') == 'plumSettleDeposit':
            action_dict = map_plum_settle_deposit(action)
        elif action.core.WhichOneof('action') == 'plumTransfer':
            action_dict = map_plum_transfer(action)
        elif action.core.WhichOneof('action') == 'depositToRewardingFund':
            action_dict = map_deposit_to_rewarding_fund(action)
        elif action.core.WhichOneof('action') == 'claimFromRewardingFund':
            action_dict = map_claim_from_rewarding_fund(action)
        elif action.core.WhichOneof('action') == 'grantReward':
            action_dict = map_grant_reward(action)
        elif action.core.WhichOneof('action') == 'stakeCreate':
            action_dict = map_stake_create(action)
        elif action.core.WhichOneof('action') == 'stakeUnstake':
            action_dict = map_stake_unstake(action)
        elif action.core.WhichOneof('action') == 'stakeWithdraw':
            action_dict = map_stake_withdraw(action)
        elif action.core.WhichOneof('action') == 'stakeAddDeposit':
            action_dict = map_stake_add_deposit(action)
        elif action.core.WhichOneof('action') == 'stakeRestake':
            action_dict = map_stake_restake(action)
        elif action.core.WhichOneof('action') == 'stakeChangeCandidate':
            action_dict = map_stake_change_candidate(action)
        elif action.core.WhichOneof('action') == 'stakeTransferOwnership':
            action_dict = map_stake_transfer_ownership(action)
        elif action.core.WhichOneof('action') == 'candidateRegister':
            action_dict = map_candidate_register(action)
        elif action.core.WhichOneof('action') == 'candidateUpdate':
            action_dict = map_candidate_update(action)
        elif action.core.WhichOneof('action') == 'putPollResult':
            action_dict = map_put_poll_result(action)

        yield {**map_base_action(raw.block, action), **map_receipt(receipt), **action_dict}

def map_base_action(block, action):
    return {
        'type': 'action',
        'version': action.core.version,
        'nonce': action.core.nonce,
        'gas_limit': to_int(action.core.gasLimit),
        'gas_price': to_int(action.core.gasPrice),
        'sender': iotex_utils.pubkey_to_address(action.senderPubKey),
        'timestamp': block.header.core.timestamp.ToJsonString()
    }

def map_transfer(action):
    transfer = action.core.transfer
    return {
        'action_type': 'transfer',
        'transfer': {
            'amount': to_int(transfer.amount),
            'recipient': transfer.recipient,
            'payload': string_utils.base64_string(transfer.payload),
        }
    }

def map_execution(action):
    execution = action.core.execution
    return {
        'action_type': 'execution',
        'execution': {
            'amount': to_int(execution.amount),
            'contract': execution.contract,
            'data': string_utils.base64_string(execution.data),
        }
    }

def map_start_sub_chain(action):
    start_sub_chain = action.core.startSubChain
    return {
        'action_type': 'start_sub_chain',
        'start_sub_chain': {
            'chain_id': start_sub_chain.chainID,
            'security_deposit': start_sub_chain.securityDeposit,
            'operation_deposit': start_sub_chain.operationDeposit,
            'start_height': start_sub_chain.startHeight,
            'parent_height_offset': start_sub_chain.parentHeightOffset
        }
    }

def map_stop_sub_chain(action):
    stop_sub_chain = action.core.stopSubChain
    return {
        'action_type': 'stop_sub_chain',
        'stop_sub_chain': {
            'chain_id': stop_sub_chain.chainID,
            'stop_height': stop_sub_chain.stopHeight,
            'sub_chain_address': stop_sub_chain.subChainAddress
        }
    }

def map_put_block(action):
    put_block = action.core.putBlock
    return {
        'action_type': 'put_block',
        'put_block': {
            'sub_chain_address': put_block.subChainAddress,
            'height': put_block.height,
            'roots': [{'name': root.name, 'value': string_utils.base64_string(root.value)} for root in put_block.roots]
        }
    }

def map_create_deposit(action):
    create_deposit = action.core.createDeposit
    return {
        'action_type': 'create_deposit',
        'create_deposit': {
            'chain_id': create_deposit.chainID,
            'amount': to_int(create_deposit.amount),
            'recipient': create_deposit.recipient,
        }
    }

def map_settle_deposit(action):
    settle_deposit = action.core.settleDeposit
    return {
        'action_type': 'settle_deposit',
        'settle_deposit': {
            'amount': to_int(settle_deposit.amount),
            'recipient': settle_deposit.recipient,
            'index': settle_deposit.index
        }
    }

def map_create_plum_chain(action):
    return {
        'action_type': 'create_plum_chain'
    }

def map_terminate_plum_chain(action):
    terminate_plum_chain = action.core.terminatePlumChain
    return {
        'action_type': 'terminate_plum_chain',
        'terminate_plum_chain': {
            'sub_chain_address': terminate_plum_chain.subChainAddress
        }
    }

def map_plum_put_block(action):
    plum_put_block = action.core.plumPutBlock
    return {
        'action_type': 'plum_put_block',
        'plum_put_block': {
            'sub_chain_address': plum_put_block.subChainAddress,
            'height': plum_put_block.height,
            'roots': [{'name': name, 'value': string_utils.base64_string(value)} for name, value in plum_put_block.roots.items()],
        }
    }

def map_plum_create_deposit(action):
    plum_create_deposit = action.core.plumCreateDeposit
    return {
        'action_type': 'plum_create_deposit',
        'plum_create_deposit': {
            'sub_chain_address': plum_create_deposit.subChainAddress,
            'amount': to_int(plum_create_deposit.amount),
            'recipient': plum_create_deposit.recipient,
        }
    }

def map_plum_start_exit(action):
    plum_start_exit = action.core.plumStartExit
    return {
        'action_type': 'plum_start_exit',
        'plum_start_exit': {
            'sub_chain_address': plum_start_exit.subChainAddress,
            'previous_transfer': string_utils.base64_string(plum_start_exit.previousTransfer),
            'previous_transfer_block_proof': string_utils.base64_string(plum_start_exit.previousTransferBlockProof),
            'previous_transfer_block_height': plum_start_exit.previousTransferBlockHeight,
            'exit_transfer': string_utils.base64_string(plum_start_exit.exitTransfer),
            'exit_transfer_block_proof': string_utils.base64_string(plum_start_exit.exitTransferBlockProof),
            'exit_transfer_block_height': plum_start_exit.exitTransferBlockHeight,
        }
    }

def map_plum_challenge_exit(action):
    plum_challenge_exit = action.core.plumChallengeExit
    return {
        'action_type': 'plum_challenge_exit',
        'plum_challenge_exit': {
            'sub_chain_address': plum_challenge_exit.subChainAddress,
            'coin_id': plum_challenge_exit.coinID,
            'challenge_transfer': string_utils.base64_string(plum_challenge_exit.challengeTransfer),
            'challenge_transfer_block_proof': string_utils.base64_string(plum_challenge_exit.challengeTransferBlockProof),
            'challenge_transfer_block_height': plum_challenge_exit.challengeTransferBlockHeight,
        }
    }

def map_plum_response_challenge_exit(action):
    plum_response_challenge_exit = action.core.plumResponseChallengeExit
    return {
        'action_type': 'plum_response_challenge_exit',
        'plum_response_challenge_exit': {
            'sub_chain_address': plum_response_challenge_exit.subChainAddress,
            'coin_id': plum_response_challenge_exit.coinID,
            'challenge_transfer': string_utils.base64_string(plum_response_challenge_exit.challengeTransfer),
            'response_transfer': string_utils.base64_string(plum_response_challenge_exit.responseTransfer),
            'response_transfer_block_proof': string_utils.base64_string(plum_response_challenge_exit.responseTransferBlockProof),
            'previous_transfer_block_height': plum_response_challenge_exit.previousTransferBlockHeight
        }
    }

def map_plum_finalize_exit(action):
    plum_finalize_exit = action.core.finalizeExit
    return {
        'action_type': 'plum_finalize_exit',
        'plum_finalize_exit': {
            'sub_chain_address': plum_finalize_exit.subChainAddress,
            'coin_id': plum_finalize_exit.coinID,
        }
    }

def map_plum_settle_deposit(action):
    plum_settle_deposit = action.core.plumSettleDeposit
    return {
        'action_type': 'plum_settle_deposit',
        'plum_settle_deposit': {
            'coin_id': plum_settle_deposit.coinID
        }
    }

def map_plum_transfer(action):
    plum_transfer = action.core.plumTransfer
    return {
        'action_type': 'plum_transfer',
        'plum_transfer': {
            'coin_id': plum_transfer.coinID,
            'denomination': string_utils.base64_string(plum_transfer.denomination),
            'owner': plum_transfer.owner,
            'recipient': plum_transfer.recipient,
        }
    }

def map_deposit_to_rewarding_fund(action):
    deposit_to_rewarding_fund = action.core.depositToRewardingFund
    return {
        'action_type': 'deposit_to_rewarding_fund',
        'deposit_to_rewarding_fund': {
            'amount': to_int(deposit_to_rewarding_fund.amount),
            'data': string_utils.base64_string(deposit_to_rewarding_fund.data),
        }
    }

def map_claim_from_rewarding_fund(action):
    claim_from_rewarding_fund = action.core.claimFromRewardingFund
    return {
        'action_type': 'claim_from_rewarding_fund',
        'claim_from_rewarding_fund': {
            'amount': to_int(claim_from_rewarding_fund.amount),
            'data': string_utils.base64_string(claim_from_rewarding_fund.data),
        }
    }

def map_grant_reward(action):
    grant_reward = action.core.grantReward
    return {
        'action_type': 'grant_reward',
        'grant_reward': {
            'type': grant_reward.type,
            'height': grant_reward.height
        }
    }

def map_stake_create(action):
    stake_create = action.core.stakeCreate
    return {
        'action_type': 'stake_create',
        'stake_create': {
            'candidate_name': stake_create.candidateName,
            'staked_amount': to_int(stake_create.stakedAmount),
            'staked_duration': stake_create.stakedDuration,
            'auto_stake': stake_create.autoStake,
            'payload': string_utils.base64_string(stake_create.payload),
        }
    }

def map_stake_unstake(action):
    stake_unstake = action.core.stakeUnstake
    return {
        'action_type': 'stake_unstake',
        'stake_unstake': {
            'bucket_index': stake_unstake.bucketIndex,
            'payload': string_utils.base64_string(stake_unstake.payload),
        }
    }

def map_stake_withdraw(action):
    stake_withdraw = action.core.stakeWithdraw
    return {
        'action_type': 'stake_withdraw',
        'stake_withdraw': {
            'bucket_index': stake_withdraw.bucketIndex,
            'payload': string_utils.base64_string(stake_withdraw.payload),
        }
    }

def map_stake_add_deposit(action):
    stake_add_deposit = action.core.stakeAddDeposit
    return {
        'action_type': 'stake_add_deposit',
        'stake_add_deposit': {
            'bucket_index': stake_add_deposit.bucketIndex,
            'amount': to_int(stake_add_deposit.amount),
            'payload': string_utils.base64_string(stake_add_deposit.payload),
        }
    }

def map_stake_restake(action):
    stake_restake = action.core.stakeRestake
    return {
        'action_type': 'stake_restake',
        'stake_restake': {
            'bucket_index': stake_restake.bucketIndex,
            'staked_duration': stake_restake.stakedDuration,
            'auto_stake': stake_restake.autoStake,
            'payload': string_utils.base64_string(stake_restake.payload),
        }
    }

def map_stake_change_candidate(action):
    stake_change_candidate = action.core.stakeChangeCandidate
    return {
        'action_type': 'stake_change_candidate',
        'stake_change_candidate': {
            'bucket_index': stake_change_candidate.bucketIndex,
            'candidate_name': stake_change_candidate.candidateName,
            'payload': string_utils.base64_string(stake_change_candidate.payload),
        }
    }

def map_stake_transfer_ownership(action):
    stake_transfer_ownership = action.core.stakeTransferOwnership
    return {
        'action_type': 'stake_transfer_ownership',
        'stake_transfer_ownership': {
            'bucket_index': stake_transfer_ownership.bucketIndex,
            'voter_address': stake_transfer_ownership.voterAddress,
            'payload': string_utils.base64_string(stake_transfer_ownership.payload),
        }
    }

def map_candidate_register(action):
    candidate_register = action.core.candidateRegister
    return {
        'action_type': 'candidate_register',
        'candidate_register': {
            'name': candidate_register.candidate.name,
            'operator_address': candidate_register.candidate.operatorAddress,
            'reward_address': candidate_register.candidate.rewardAddress,
            'staked_amount': to_int(candidate_register.stakedAmount),
            'staked_duration': candidate_register.stakedDuration,
            'auto_stake': candidate_register.autoStake,
            'owner_address': candidate_register.ownerAddress,
            'payload': string_utils.base64_string(candidate_register.payload),
        }
    }

def map_candidate_update(action):
    candidate_update = action.core.candidateUpdate
    return {
        'action_type': 'candidate_update',
        'candidate_update': {
            'name': candidate_update.name,
            'operator_address': candidate_update.operatorAddress,
            'reward_address': candidate_update.rewardAddress,
        }
    }

def map_put_poll_result(action):
    put_poll_result = action.core.putPollResult
    candidates = [map_candidate(candidate) for candidate in put_poll_result.candidates.candidates]
    return {
        'action_type': 'put_poll_result',
        'put_poll_result': {
            'height': put_poll_result.height,
            'candidates': candidates,
        }
    }

def map_candidate(candidate):
    return {
        'address': candidate.address,
        'votes': string_utils.base64_string(candidate.votes),
        'pub_key': string_utils.base64_string(candidate.pubKey),
        'reward_address': candidate.rewardAddress
    }


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/types/node.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/types/node.proto',
  package='iotextypes',
  syntax='proto3',
  serialized_options=b'\n\"com.github.iotexproject.grpc.typesP\001Z5github.com/iotexproject/iotex-proto/golang/iotextypes',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16proto/types/node.proto\x12\niotextypes\"v\n\nServerMeta\x12\x16\n\x0epackageVersion\x18\x01 \x01(\t\x12\x17\n\x0fpackageCommitID\x18\x02 \x01(\t\x12\x11\n\tgitStatus\x18\x03 \x01(\t\x12\x11\n\tgoVersion\x18\x04 \x01(\t\x12\x11\n\tbuildTime\x18\x05 \x01(\tB]\n\"com.github.iotexproject.grpc.typesP\x01Z5github.com/iotexproject/iotex-proto/golang/iotextypesb\x06proto3'
)




_SERVERMETA = _descriptor.Descriptor(
  name='ServerMeta',
  full_name='iotextypes.ServerMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='packageVersion', full_name='iotextypes.ServerMeta.packageVersion', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='packageCommitID', full_name='iotextypes.ServerMeta.packageCommitID', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gitStatus', full_name='iotextypes.ServerMeta.gitStatus', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='goVersion', full_name='iotextypes.ServerMeta.goVersion', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='buildTime', full_name='iotextypes.ServerMeta.buildTime', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=156,
)

DESCRIPTOR.message_types_by_name['ServerMeta'] = _SERVERMETA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ServerMeta = _reflection.GeneratedProtocolMessageType('ServerMeta', (_message.Message,), {
  'DESCRIPTOR' : _SERVERMETA,
  '__module__' : 'proto.types.node_pb2'
  # @@protoc_insertion_point(class_scope:iotextypes.ServerMeta)
  })
_sym_db.RegisterMessage(ServerMeta)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
